#!/usr/bin/env python
# -*- coding: utf-8 -*-

import sys
import os
import pwd
import datetime
import ldap
import commands
import socket
import logging
import grp
import string
import subprocess
from subprocess import Popen
from pwd import getpwnam


from crypt import crypt
from PyQt4 import QtGui, QtCore
from PyQt4.QtGui import *

from cgauserconfig import CUSTOMUSERSFILE
from cgauserconfig import DEFAULTGROUPS



class MissingUserData(Exception): pass
class CgaUserExists(Exception): pass
class CgaUserCreationError(Exception): pass
class CgaUserLDAPInvalidCredentials(Exception): pass


class CgaUser:

    def __init__(self, login, nombre, passwd):
        self.id, self.nombre, self.password = login, nombre, passwd

    def save(self):
        self._save()

    def _save(self):
        self._add_to_system()
        self._add_to_list()
        self.sync()

    def _get_new_uid(self):
        validos = [ t.pw_uid for t in pwd.getpwall() if 1000 <= t.pw_uid < 10000 ]
        validos.sort()
        return validos[-1]+1

    def checkUser(self):
        try:
            user=pwd.getpwnam(self.id)
            raise CgaUserExists("El usuario %s ya existe" % self.id)
        except KeyError:
            pass
        # El usuario no existe, podemos proceder.

        self.login()

    def _add_to_system(self):
        # Se crea el usuario
        # Primero se cifra el password para pasarlo a adduser
        try:
            passwd = crypt(self.password.encode('utf8'), "00")
            uid=self._get_new_uid()

            # Cambio 7173: Cambio de los permisos de los usuarios locales.
            umask="077"

            groupinfo = grp.getgrnam(self.grupo)
            gid = groupinfo.gr_gid
            comando = "/usr/sbin/useradd -m -G {grupos} -p {password} -u {uid} -g {gid} -s /bin/bash -K UMASK={umask} {usuario}"
            comando = comando.format(   grupos=DEFAULTGROUPS,
                                        password=passwd,
                                        usuario=self.id,
                                        uid=uid,
                                        gid=gid,
                                        umask=umask)
            #print "Ejecutando '%s'" % comando

            (ret, salida)=commands.getstatusoutput(comando)
            comando=None
            if ret != 0:
                raise CgaUserCreationError("Error al crear el usuario: '%s'" % salida)


            comando='/usr/bin/chfn -f "'+ self.nombre +'"' + ' ' + self.id
            comando = comando.encode('utf8')

            (ret, salida)=commands.getstatusoutput(comando)
            comando=None
            if ret!=0:
                # Si el comando anterior falla tampoco pasa nada, es información optativa
                print "No se pudo establecer el GECOS: %s" % salida


            comando = ('echo "{username}:{password}" | chpasswd'.format(
                                        username=self.id,
                                        password=self.password))

            (ret, salida)=commands.getstatusoutput(comando)
            comando=None
            if ret!=0:
                print "Error al modificar la contraseña."


        except KeyError as e:
            print e
            raise CgaUserCreationError("Ha ocurrido un error al crear el usuario.")
        except CgaUserCreationError:
            raise CgaUserCreationError("Ha ocurrido un error al crear el usuario.")
        except Exception as e:
            print e

    def functionTic(self):
        login = self.id
        uid = getpwnam(login).pw_uid
        gid = getpwnam(login).pw_gid
        HOMEDIR = os.path.join("/home/" + login + os.path.sep)
        PATHFICH = '/usr/lib/cga/ticboxcloud-configurator'
        fichero = 'owncloud.cfg'
        ficheroAuto = 'TicBoxCloud.desktop'
        ruta = os.path.join(HOMEDIR + '.local/share/data/TicBoxCloud')
        ruta2 = os.path.join(HOMEDIR + '.local')
        rutaAuto = os.path.join(HOMEDIR + '.config/autostart')
        rutaAuto2 = os.path.join(rutaAuto + os.path.sep + ficheroAuto)

        if os.path.exists(HOMEDIR):
            process = subprocess.Popen(['chown', '-R', str(uid) + ':' + str(gid), HOMEDIR])

        if not os.path.exists(rutaAuto):
            os.mkdir(rutaAuto, 755)

        sentCpAuto = "cp '"+PATHFICH+os.path.sep+ficheroAuto+"' "+rutaAuto
        os.system(sentCpAuto)
        process = subprocess.Popen(['chmod', '0775', rutaAuto])
        process = subprocess.Popen(['chmod', '0661', rutaAuto2])
        process = subprocess.Popen(['chown', '-R', str(uid) + ':' + str(gid), rutaAuto])

        if not os.path.exists(ruta):
            os.makedirs(ruta, mode = 0775)

        sentCp = "cp '"+PATHFICH+os.path.sep+fichero+"' "+ruta
        os.system(sentCp)

        lineas = ["0\Folders\TicBoxCloud\localPath=" + HOMEDIR + "TicBoxCloud"+ os.path.sep, "0\http_user=" + login, "0\user=" + login]
        path = os.path.join(ruta,str(fichero))
        if os.path.exists(os.path.abspath(path)):
            self.archivo = open(path, 'a+')
            for l in lineas:
                self.archivo.write(l + '\n')
            self.archivo.close()
            process = subprocess.Popen(['chmod', '-R', '0775', ruta2])
            proceso = subprocess.Popen(['chown', '-R', str(uid) + ':' + str(gid), ruta2])

        pathTic = os.path.join(HOMEDIR + "TicBoxCloud")
        if not os.path.exists(os.path.abspath(pathTic)):
            os.mkdir(pathTic, 0775)
            process = subprocess.Popen(['chown', '-R', str(uid) + ':' + str(gid), pathTic])



    def _add_to_list(self):
        usuario = "root"
        fecha=str(datetime.datetime.now())
        with open(CUSTOMUSERSFILE, "a") as f:
            f.write("%s:%s:%s\n" % (usuario, fecha, self.id))



    def login(self):
        ficheroentrada = open('/etc/resolv.conf','r')
        palabra = "search"
        for linea in ficheroentrada:
            if palabra in linea:
                domain=linea.split(" ")[1].split(".")[0]
                provincia=linea.split(" ")[1].split(".")[1]
        ficheroentrada.close()


        #Se comprueba que el usuario a crear existe en ldap
        if self.id != " " and self.password != "" :
            LDAP_SERVER = "ldap://c0:389"
            groupdict={'10000':'alumnos','10001': 'profesores','10002': 'gestion'}
            l = ldap.initialize(LDAP_SERVER)
            username = "uid=%s,ou=People,dc=%s,dc=%s,dc=andared,dc=ced,dc=junta-andalucia,dc=es" % (self.id, domain, provincia)
            password = self.password

            try:
                l.simple_bind_s(username, password)
                baseDN = "ou=People,dc=%s,dc=%s,dc=andared,dc=ced,dc=junta-andalucia,dc=es"  % (domain, provincia)
                print baseDN
                searchScope = ldap.SCOPE_SUBTREE
                ## retrieve all atributes - again adjust to your needs - see documentation for more options
                retrieveAttributes = None
                searchFilter = "(uid="+self.id+")"

                error = False
                try:
                    ldap_result_id = l.search(baseDN, searchScope, searchFilter, retrieveAttributes)
                    result_set = []
                    while 1:
                        result_type, result_data = l.result(ldap_result_id, 0)
                        if result_data == []:
                            break
                        else:
                            ## here you don't have to append to a list
                            ## you could do whatever you want with the individual entry
                            ## The appending to list is just for illustration.
                            if result_type == ldap.RES_SEARCH_ENTRY:
                                result_set.append(result_data)

                    if result_set[0][0][1]['gidNumber'][0] in groupdict:
                        self.grupo = groupdict[str(result_set[0][0][1]['gidNumber'][0])]
                    else:
                        print "El grupo no existe"


                except ldap.LDAPError, e:
                    print "Exception LDAP: %s "%e


            except ldap.INVALID_CREDENTIALS as e:
                error = True
                raise CgaUserLDAPInvalidCredentials("Autenticacion usuario LDAP incorrecta")
            except ldap.SERVER_DOWN as e:
                error = True
                raise CgaUserLDAPInvalidCredentials("Autenticacion usuario LDAP incorrecta")
            except Exception as e:
                error = True
                raise CgaUserLDAPInvalidCredentials("Autenticacion usuario LDAP incorrecta")

            if error !=True:
                self.save()



    def sync(self):
        try:
            command = "TBCLOUD:"+self.id+":"
            host = "c0"
            port = 30000

            self.__send_command(command, host, port)

        except Exception as e:
            print "Exception sync: %s " %e
            #QtGui.QMessageBox.information(self.w, u"Error", u"Ha ocurrido un error al sincronizar el directorio.")



    def __send_command(self, command, host, port):
        res = 1
        tries = 3
        while ( tries > 0 ):
            try:
                s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
                s.connect((host, port))
                logging.debug("Sent from %s" %( repr(command)))
                returned = s.send(command)
                #logging.debug("The call send returns %s (bytes sent?)" %repr(returned))
                res = 0
                break
            except Exception as e:
                logging.error( "Error sending command %s to %s:%s \nReason: %s"
                            %(command, host, port, str(e)))
            finally:
                s.close()
                tries -= 1
        if( tries==0 and res!=0):
            logging.error("After 3 tries the request can't be sent %s:%s" %(host,port))
        return res




if __name__=="__main__":
    c = CgaUser()
    c.save()
